/************************************************************************\
* custom.cpp - A custom formatting library for foobar2000's              *
* foo_playlist_ouput library.                                            *
* Copyright (c) 2005 Brian A. Hartvigsen                                 *
* Some right reserved                                                    *
**************************************************************************
* This work is licensed under the Creative Commons                       *
* Attribution-NonCommercial License. To view a copy of this license,     *
* visit http://creativecommons.org/licenses/by-nc/2.0/ or send a letter  *
* to Creative Commons, 559 Nathan Abbott Way, Stanford, California       *
* 94305, USA.                                                            *
\************************************************************************/
#pragma warning(disable:4267 4311 4312)
#include "../SDK/foobar2000.h"
#include "../foosion_helpers/extra_items_helper.h"
#pragma warning(default:4267 4311 4312)

#include "../foo_playlist_output/output_format.h"
#include "resource.h"

static cfg_string	cfg_header("header", "");
static cfg_string	cfg_footer("footer", "");
static cfg_string	cfg_format("format", "");
static cfg_int		cfg_autocrlf("autocrlf", 1);

#define CUSTOM_NAME		"Custom Playlist Output"
#define CUSTOM_VERSION	"1.03.00"

class format_custom : public output_format
{
	extra_items_helper eih;

public:
	static void htmlsafe(string8 & title)
	{
		int pos;
		while((pos = title.find_first("@htmlsafe{", 0)) != -1)
		{
			int end = title.find_first('}', pos);
			if(end == -1)
				break;

			string8 before, after, data;
			data = title.get_ptr() + pos + 10;
			data.truncate(end - pos - 10);
			after = title.get_ptr() + end + 1;
			before = title;
			before.truncate(pos);

			char *target[] = {
				"&",
				">",
				"<",
				NULL
			};

			char *replace[] = {
				"&amp;",
				"&gt;",
				"&lt;"
			};

			for(int i = 0; target[i] != NULL; i++)
			{
				int sub = 0;
				while((sub = data.find_first(target[i], sub)) != -1)
				{
					data.remove_chars(sub, 1);
					data.insert_chars(sub, replace[i]);
					sub += lstrlen(replace[i]);
				}
			}

			title = before;
			title += data;
			title += after;
		}
	}

	virtual void get_name(string_base & out)
	{
		out = "Custom";
	}

	virtual bool get_description(string_base & out)
	{
		out = "Custom formatting strings as defined in prefs.";
		return true;
	}

	virtual void get_file_mask(string_base & out)
	{
		out = "All Files|*.*";
	}

	virtual bool get_formating_string(string_base & out)
	{
		out = cfg_format;
		if(cfg_format.is_empty())
			return false;
		return true;
	}

	virtual bool get_pre_output(string_base & out)
	{
		string8 outPre;
		
		if(eih.get_count() > 0)
			eih.delete_all();

		double total = 0, test = 0;
		unsigned long nolength = 0;
		metadb_handle_list temp;

		playlist_oper::get()->get_data(temp);

		for(unsigned i = 0; i < temp.get_count(); i++)
			if((test = temp.get_item(i)->handle_get_length()) > 0)
				total += test;
			else
				nolength++;
		temp.delete_all();

		double average_sec = total / (double)playlist_oper::get()->get_count();

		#pragma warning(disable:4244)
		eih.add_item("average_track_length_string", (const char *)string_print_time(average_sec));
		eih.add_item("playlist_length_string", (const char *)string_print_time(total));

		unsigned int average_min, total_hours, total_minutes;

		average_min = average_sec / 60;
		average_sec -= average_min * 60;

		total_hours = total / 3600;
		total -= total_hours * 3600;

		total_minutes = total / 60;
		total -= total_minutes * 60;

		eih.add_integer("average_track_length_seconds", average_sec);
		eih.add_double("average_track_length_seconds_d", average_sec);
		eih.add_integer("average_track_length_minutes", average_min);
		eih.add_integer("playlist_length_seconds", total);
		eih.add_double("playlist_length_seconds_d", total);
		eih.add_integer("playlist_length_minutes", total_minutes);
		eih.add_integer("playlist_length_hours", total_hours);
		eih.add_integer("playlist_total", playlist_oper::get()->get_count());
		eih.add_integer("num_tracks_no_length", nolength);
		eih.add_item("custom_output_version", CUSTOM_NAME " " CUSTOM_VERSION);
		#pragma warning(default:4244)

		playlist_oper::get()->format_title(0, outPre, cfg_header, eih.get_ptr());
		

		if(outPre.length() < 0)
			return false;

		htmlsafe(outPre);
		out = outPre;

		if(cfg_autocrlf)
			out += "\r\n";

		return true;
	}

	virtual bool get_post_output(string_base & out)
	{
		string8 temp;
		playlist_oper::get()->format_title(0, temp, cfg_footer, eih.get_ptr());
		eih.delete_all();

		if(temp.length() < 0)
			return false;
		

		htmlsafe(temp);
		out = temp;

		if(cfg_autocrlf)
			out += "\r\n";
		return true;
	}

	virtual void run(string8 & title, int/* titleLength*/, int/* needLength*/)
	{
		/* you can do any custom processing you want here
		*   titleformat::remove_color_marks(title.get_ptr(), temp);
		* or what ever..  playlist_output_foobar does a lot of regular
		* expression stuff using PCRE... 
		* What ever..  The playlist formatting is applied before the title
		* is sent here..  Anything else?  Nope..  Think we are good..
		*/

		htmlsafe(title);

		if(cfg_autocrlf)
			title += "\r\n";
	}
};

class config_custom : public config
{
private:
	static bool read_string_from_file(reader *p_file, cfg_string &data)
	{
		WORD size;
		char *p;

		if (p_file->read_word_le(size) == 0)
			return false;

		p = new char[ size + 1];
		ZeroMemory(p, size + 1);

		if (p_file->read(p,size) == 0)
			return false;

		data = p;
		delete[] p;
		return true;
	}

	static void write_string_to_file(reader *p_file, cfg_string &data)
	{
		#pragma warning(disable:4267)
		p_file->write_word_le(strlen(data.get_val()));
		p_file->write(data.get_val(), strlen(data.get_val()));
		#pragma warning(default:4267)
	}

	static void update_dialog(HWND wnd)
	{
		uSetDlgItemText(wnd, IDC_HEADER, cfg_header);
		uSetDlgItemText(wnd, IDC_FOOTER, cfg_footer);
		uSetDlgItemText(wnd, IDC_FORMAT, cfg_format);
		CheckDlgButton(wnd, IDC_AUTOCRLF, cfg_autocrlf);
	}

public:
	static LRESULT CALLBACK EditProc(HWND wnd, UINT msg,WPARAM wp,LPARAM lp)
	{
		#pragma warning(disable:4312)
		WNDPROC oldProc = (WNDPROC)GetWindowLong(wnd, GWL_USERDATA);
		#pragma warning(default:4312)

		if (msg == WM_KEYDOWN && wp == 'A')
		{
			if ((GetAsyncKeyState(VK_CONTROL)&0x8000) && (!(GetAsyncKeyState(VK_MENU)&0x8000)))
			{
				// Ctrl-A pressed. Select all text in the Edit control
				SendMessage(wnd, EM_SETSEL, 0, -1);
				return 0;
			}
		}
		return CallWindowProc(oldProc, wnd, msg, wp, lp);
	}

	static BOOL CALLBACK CustomProc(HWND wnd,UINT msg,WPARAM wp,LPARAM lp)
	{
		switch(msg)
		{
		case WM_INITDIALOG:
			{
				WNDPROC oldProc = NULL;
			
				update_dialog(wnd);

				#pragma warning(disable:4312 4311)
				oldProc = (WNDPROC)SetWindowLong(GetDlgItem(wnd, IDC_HEADER), GWL_WNDPROC, (LONG)EditProc);
				SetWindowLong(GetDlgItem(wnd, IDC_HEADER), GWL_USERDATA, (LONG)oldProc);

				oldProc = (WNDPROC)SetWindowLong(GetDlgItem(wnd, IDC_FOOTER), GWL_WNDPROC, (LONG)EditProc);
				SetWindowLong(GetDlgItem(wnd, IDC_FOOTER), GWL_USERDATA, (LONG)oldProc);

				oldProc = (WNDPROC)SetWindowLong(GetDlgItem(wnd, IDC_FORMAT), GWL_WNDPROC, (LONG)EditProc);
				SetWindowLong(GetDlgItem(wnd, IDC_FORMAT), GWL_USERDATA, (LONG)oldProc);
				#pragma warning(default:4312 4311)
			}
			return 1;

		case WM_COMMAND:
			switch(wp)
			{
			case BN_CLICKED<<16|IDC_CONFIG_RESET:
				cfg_header = "";
				cfg_footer = "";
				cfg_format = "";
				cfg_autocrlf = BST_CHECKED;
				
				update_dialog(wnd);
				break;
			case BN_CLICKED<<16|IDC_CONFIG_LOAD:
				{
					string8 fn;

					if (!uGetOpenFileName(wnd,"Config Files (*.poc)|*.poc",0,"poc","Load Configuration", NULL, fn, FALSE))
						break;

					reader *p_reader = file::g_open(fn, reader::MODE_READ);
					if (p_reader == NULL)
					{
						console::error(uStringPrintf("Error opening config file: \"%s\"", fn.get_ptr()));
						break;
					}

					if (!read_string_from_file(p_reader, cfg_header) || 
						!read_string_from_file(p_reader, cfg_format) ||
						!read_string_from_file(p_reader, cfg_footer))
					{
						console::error(uStringPrintf("Invalid config file: \"%s\"", fn));
						p_reader->service_release();
						break;
					}

					bool temp = true;
					p_reader->read_x_le(temp);
					p_reader->service_release();
					cfg_autocrlf = temp;

					update_dialog(wnd);
				}
				break;
			case BN_CLICKED<<16|IDC_CONFIG_SAVE:
				{
					string8 fn;

					if (!uGetOpenFileName(wnd,"Config Files (*.poc)|*.poc",0,"poc","Save Configuration", NULL, fn, TRUE))
						break;

					reader *p_reader = file::g_open(fn, reader::MODE_WRITE_NEW);
					if (p_reader == NULL)
					{
						console::error(uStringPrintf("Can not save to file: \"%s\"", fn.get_ptr()));
						break;
					}

					write_string_to_file(p_reader, cfg_header);
					write_string_to_file(p_reader, cfg_format);
					write_string_to_file(p_reader, cfg_footer);

					#pragma warning(disable:4800)
					bool temp = cfg_autocrlf;
					#pragma warning(default:4800)

					p_reader->write_x_le(temp);
					p_reader->service_release();
				}
				break;
			case BN_CLICKED<<16|IDC_AUTOCRLF:
				cfg_autocrlf = (int)uSendMessage((HWND)lp,BM_GETCHECK,0,0);
				break;
			case EN_CHANGE<<16|IDC_HEADER:
				cfg_header =  string_utf8_from_window((HWND)lp);
				break;
			case EN_CHANGE<<16|IDC_FOOTER:
				cfg_footer = string_utf8_from_window((HWND)lp);
				break;
			case EN_CHANGE<<16|IDC_FORMAT:
				cfg_format = string_utf8_from_window((HWND)lp);
				break;
			}
			break;
		}

		return 0;
	}

	virtual HWND create(HWND parent)
	{
		return uCreateDialog(IDD_CONFIG_CUSTOM,parent,CustomProc);
	}
	virtual const char * get_name() {return "Custom";}
	virtual const char * get_parent_name() {return "Playlist Output";}
};

static service_factory_single_t<config, config_custom> foo4;
static output_format_factory<format_custom> foobar;